#version 430

out vec4 FragColor;

in vec2 texCoords;

uniform int windowWidth;
uniform int windowHeight;
uniform float brightness;
uniform bool celShading;
uniform bool showContour;

uniform sampler2D screenTexture;

const int CEL_SHADING_LEVELS = 10;

vec3 rgb2hsv(vec3 rgb)
{ 
    // h, s, v = hue, saturation, value
    float cmax = max(rgb.r, max(rgb.g, rgb.b));
    float cmin = min(rgb.r, min(rgb.g, rgb.b));
    float diff = cmax - cmin;   
    
    float h, s;

    if (diff == 0.0)
        h = 0;
    else if (cmax == rgb.r) 
		h = int(60 * ((rgb.g - rgb.b) / diff) + 360) % 360;
    else if (cmax == rgb.g)
        h = int(60 * ((rgb.b - rgb.r) / diff) + 120) % 360;
    else if (cmax == rgb.b)
        h = int(60 * ((rgb.r - rgb.g) / diff) + 240) % 360;

    if (cmax == 0)
        s = 0;
    else
        s = (diff / cmax) * 100;
  
    float v = cmax * 100;

	return vec3(h, s, v);
}

void main()
{
	vec3 color = texture(screenTexture, texCoords).rgb;

	FragColor = vec4(color, 1.0f); 

	bool isContour = false;
	if (showContour) {
		vec4 n[9];

		float w = 1.0 / windowWidth;
		float h = 1.0 / windowHeight;

		n[0] = texture2D(screenTexture, texCoords + vec2( -w, -h));
		n[1] = texture2D(screenTexture, texCoords + vec2(0.0, -h));
		n[2] = texture2D(screenTexture, texCoords + vec2(  w, -h));
		n[3] = texture2D(screenTexture, texCoords + vec2( -w, 0.0));
		n[4] = texture2D(screenTexture, texCoords);
		n[5] = texture2D(screenTexture, texCoords + vec2(  w, 0.0));
		n[6] = texture2D(screenTexture, texCoords + vec2( -w, h));
		n[7] = texture2D(screenTexture, texCoords + vec2(0.0, h));
		n[8] = texture2D(screenTexture, texCoords + vec2(  w, h));
	
		vec4 sobel_edge_h = n[2] + (2.0*n[5]) + n[8] - (n[0] + (2.0*n[3]) + n[6]);
  		vec4 sobel_edge_v = n[0] + (2.0*n[1]) + n[2] - (n[6] + (2.0*n[7]) + n[8]);
		vec4 sobel = sqrt((sobel_edge_h * sobel_edge_h) + (sobel_edge_v * sobel_edge_v));

		if (length(sobel.rgb) > 0.8) {
		    FragColor = vec4(vec3(0), 1.0);
		    isContour = true;
		}
	}

	if (! isContour && celShading) {
		vec3 hsv = rgb2hsv(color);
		float level = floor(hsv.z * CEL_SHADING_LEVELS / 100.0);
		float coeff = 10.0 * level + 5.0;
		FragColor = vec4(color * coeff / hsv.z, 1.0f); 
	}
}