#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;
uniform float brightness;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;

uniform vec3 textureColor;

const int CEL_SHADING_LEVELS = 4;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
	bool enabled;
} dirL;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha) {
	float d = length(l);
	l = normalize(l);

	float diffuse = max(0, dot(n, l));
	float level = floor(diffuse * CEL_SHADING_LEVELS);
	diffuse = level / CEL_SHADING_LEVELS;

	vec3 r = reflect(-l, n);
	float specular = pow(max(0, dot(r, v)), alpha);
	level = floor(specular * CEL_SHADING_LEVELS);
	specular = level / CEL_SHADING_LEVELS;

	return diffuseF * diffuseC * diffuse + specularF * specularC * specular; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	color = vec4(textureColor * materialCoefficients.x, 1); // ambient
	
	if (dirL.enabled) {
		color.rgb += phong(n, -dirL.direction, v, dirL.color * textureColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha);
	}

	color.rgb *= max(0.1, brightness);
}

